/***************************************************************************
                 mondo.c  -  the place where everything starts
                             -------------------
    begin                : Wed Dec 12 07:05:40 CST 2001
    copyright            : (C) 2001 by Kelledin
    email                : kelledin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of version 2 of the GNU General Public License as  *
 *   published by the Free Software Foundation.                            *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "mondo.h"

const char *opt_string = "c:s:dhlV";

int main(int argc, char **argv)
{
    int          opt_char;
    int          daemonize    = 1;
    const char  *sensors_file = NULL;

    do {
        opt_char=getopt(argc, argv, opt_string);

        switch(opt_char) {
        case -1:
            break;

        case 'd':
            daemonize=0;
            break;

        case 'c':
            if (conf_file!=NULL) {
                fprintf(stderr, "FATAL: '-c' option specified twice!\n");
                return -1;
            }

            conf_file=optarg;
            break;

        case 's':
            if (sensors_file!=NULL) {
                fprintf(stderr, "FATAL: '-s' option specified twice!\n");
                return -1;
            }

            sensors_file=optarg;
            break;

        case 'h':
            printf("USAGE: %s [-dhV] [-c config_file] [-s sensors_file]\n", *argv);
            printf("  -c config_file  : specify a file to read configuration info from\n");
            printf("                    (default is %s)\n", MONDO_CONF_FILE);
            printf("  -s sensors_file : specify the location of lm_sensors sensors.conf\n");
            printf("                    (default is %s)\n", SENSORS_CONF_FILE);
            printf("  -d              : do not daemonize\n");
            printf("  -h              : print out help (this screen)\n");
            printf("  -l              : do not create/lock a pidfile.\n");
            printf("  -V              : print out version.\n\n" );
            return 0;

        case 'l':
            dolock=0;
            break;

        case 'V':
            printf("System Monitor Daemon v%s\n\n", MONDO_VERSION);
            return 0;

        case '?':
            fprintf(stderr, "USAGE: %s [-dhV] [-c config_file] [-s sensors_file]\n", *argv);
            fprintf(stderr, "  -c config_file  : specify a file to read configuration info from\n");
            fprintf(stderr, "                    (default is %s)\n", MONDO_CONF_FILE);
            fprintf(stderr, "  -s sensors_file : specify the location of lm_sensors sensors.conf\n");
            fprintf(stderr, "                    (default is %s)\n", SENSORS_CONF_FILE);
            fprintf(stderr, "  -d              : do not daemonize\n");
            fprintf(stderr, "  -h              : print out help (this screen)\n");
            fprintf(stderr, "  -l              : do not create/lock a pidfile.\n");
            fprintf(stderr, "  -V              : print out version.\n\n" );
            return -1;

        default:
            fprintf(stderr, "Invalid option: -%c\n", opt_char);
            fprintf(stderr, "USAGE: %s [-dhV] [-c config_file] [-s sensors_file]\n", *argv);
            fprintf(stderr, "  -c config_file  : specify a file to read configuration info from\n");
            fprintf(stderr, "                    (default is %s)\n", MONDO_CONF_FILE);
            fprintf(stderr, "  -s sensors_file : specify the location of lm_sensors sensors.conf\n");
            fprintf(stderr, "                    (default is %s)\n", SENSORS_CONF_FILE);
            fprintf(stderr, "  -d              : do not daemonize\n");
            fprintf(stderr, "  -h              : print out help (this screen)\n");
            fprintf(stderr, "  -l              : do not create/lock a pidfile.\n");
            fprintf(stderr, "  -V              : print out version.\n\n" );
            return -1;
        }
    } while (opt_char!=-1);

    if (conf_file==NULL) {
        conf_file=MONDO_CONF_FILE;
    }

    if (sensors_file==NULL) {
        sensors_file=SENSORS_CONF_FILE;
    }

    if (init_lm_api(sensors_file)<0) {
        fprintf(stderr, "FATAL: initializing lm_sensors: %s\n",
                strerror(errno));

        return -1;
    }

    /* Open the syslog and log a startup message. */
    openlog("mondo", LOG_CONS|LOG_PID, LOG_DAEMON);
    syslog(LOG_INFO, "System Monitor v%s started.", MONDO_VERSION);

    /* Set up the signal handlers and read mondo.conf. */
    if (set_signal_handlers()<0 || read_conf()<0) {
        shutdown_lm_api();
        return -1;
    }

    if (daemonize) {
        /* Go into daemon mode. */
        if (daemon(0, 0)<0) {
            fprintf(stderr, "FATAL: trying to daemonize: %s\n",
                    strerror(errno));

            shutdown_lm_api();
            return -1;
        }

        if (dolock==1 && lock_pidfile()<0) {
            shutdown_lm_api();
            return -1;
        }
    }

    /* Start the alarm ticker. */
    if (set_alrm_timer(mondo_delay)<0) {
        syslog(LOG_ERR, "setting alarm timer: %s", strerror(errno));
        syslog(LOG_ERR, "exiting due to previous errors.");

        if (dolock) {
            unlock_pidfile();
        }

        shutdown_lm_api();
        return -1;
    }

    /* Go into a pause loop. */
    for ( ; ; ) {
        pause();
    }

    /* Put a return here to make the compiler happy, even though we should
     * never reach this point.
     */
    return 0;
}
